<?php

declare(strict_types=1);

namespace Drupal\billwerk_subscriptions\Form;

use Drupal\billwerk_subscriptions\Environment;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure Billwerk Subscriptions settings for this site.
 */
final class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'billwerk_subscriptions_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['billwerk_subscriptions.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form = parent::buildForm($form, $form_state);
    $config = $this->config('billwerk_subscriptions.settings');

    $form['settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Settings'),
      '#description' => $this->t('General Billwerk Subscriptions settings'),
      '#description_display' => 'after',
    ];
    $form['settings']['environment'] = [
      '#type' => 'radios',
      '#title' => $this->t('Billwerk Environment'),
      '#options' => Environment::getEnvironments(),
      '#default_value' => $config->get('environment'),
      '#description' => $this->t('Select the environment to use.'),
    ];
    $roles_options = user_role_names(TRUE);
    // Remove options that should never be selectable:
    unset($roles_options['authenticated'], $roles_options['authenticated']);
    $form['settings']['billwerk_roles'] = [
			'#type' => 'checkboxes',
			'#title' => $this->t('Billwerk roles'),
      '#options' => array_map('\Drupal\Component\Utility\Html::escape', $roles_options),
			'#default_value' => $config->get('billwerk_roles'),
			'#description' => $this->t('Select the roles that should be (un)assigned by Billwerk Subscriptions. <strong>Important:</strong> Only use <em>dedicated roles</em> and <em>never (un)assign these manually</em> in Drupal, as this will be overwritten!'),
		];
    $form['settings']['logging_levels'] = [
      '#type' => 'radios',
      '#title' => $this->t('Logging levels'),
      '#options' => [
        'emergency' => $this->t('Emergency'),
        'alert' => $this->t('Alert'),
        'critical' => $this->t('Critical'),
        'error' => $this->t('Error'),
        'warning' => $this->t('Warning'),
        'notice' => $this->t('Notice'),
        'info' => $this->t('Info'),
        'debug' => $this->t('Debug'),
      ],
      '#default_value' => $config->get('logging_levels'),
      '#description' => $this->t('Select the logging levels to use.'),
    ];
    $form['settings']['api_response_cache_enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Cache API responses'),
      '#default_value' => $config->get('api_response_cache_enabled'),
      '#description' => $this->t('Cache API responses during script execution <em>(drupal_static())</em>. Should typically be enabled to reduce API calls.'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $this->config('billwerk_subscriptions.settings')
      ->set('environment', $form_state->getValue(['settings', 'environment']))
      ->set('billwerk_roles', $form_state->getValue(['settings', 'billwerk_roles']))
      ->set('logging', $form_state->getValue(['settings', 'logging']))
      ->set('api_response_cache_enabled', $form_state->getValue(['settings', 'api_response_cache_enabled']))
      ->save();
    parent::submitForm($form, $form_state);
  }

}
